@@ -26,7 +26,7 @@ struct {
     int armor;
     int compress;
     char *outfile;
-    int reserved0;
+    int textmode;
     int batch;	    /* run in batch mode */
     int answer_yes; /* answer yes on most questions */
     int answer_no;  /* answer no on most questions */
@@ -35,7 +35,7 @@ struct {
     int fingerprint; /* list fingerprints */
     int list_sigs;   /* list signatures */
     int no_armor;
-    int reserved5;
+    int list_packets; /* list-packets mode */
     int reserved6;
     int reserved7;
     int reserved8;
